import asyncio
import re
import aiosqlite
import aiohttp
from telegram import Update, MessageEntity
from telegram.ext import Application, MessageHandler, filters

# Constants
BOT_TOKEN = '7811139076:AAEAGl2cu4c0HP9MN0JHGtsBafmadb5BcFs'  # Replace with your Telegram Bot token
GROUP_ID = -1002596386377
TRACKED_TOPICS = [876, 868, 870, 909, 874, 878]
ALERT_TOPIC_ID = 34601
DB_FILE = 'tokens.db'
CHECK_INTERVAL = 2  # Changed to 2 seconds between market cap checks
DEXSCREENER_API = "https://api.dexscreener.com/latest/dex/tokens/{contract}"
REFERRAL_ID = 'rLkfkJiz_'  # Referral ID to remove from contract address

# Initialize the database
async def init_db():
    print("[DEBUG] Initializing database...")
    async with aiosqlite.connect(DB_FILE) as db:
        await db.execute('''
            CREATE TABLE IF NOT EXISTS tokens (
                contract_address TEXT PRIMARY KEY,
                name TEXT,
                ticker TEXT,
                initial_market_cap FLOAT,
                first_topic TEXT,
                chart_link TEXT,
                buy_link TEXT,
                message_link TEXT,
                first_seen_timestamp REAL,  -- Store Unix timestamp as float
                last_alerted_multiple INTEGER DEFAULT 1
            )
        ''')
        await db.commit()
    print("[DEBUG] Database initialized successfully")

# Format market cap for display
def format_market_cap(value):
    if value >= 1000000:
        formatted = f"${value / 1000000:.1f}M"
    elif value >= 1000:
        formatted = f"${value / 1000:.1f}k"
    else:
        formatted = f"${value:.0f}"
    return formatted.replace('.0', '')

# Parse message to extract token information and links
def parse_message(text, entities):
    print("[DEBUG] Parsing message...")
    ticker_match = re.search(r'purchased (.+?) @', text)
    if not ticker_match:
        print("[DEBUG] Ticker not found")
        return None
    ticker = ticker_match.group(1)
    print(f"[DEBUG] Ticker found: {ticker}")

    name = re.sub(r'^[#\$]', '', ticker).strip()
    print(f"[DEBUG] Token name: {name}")

    # Extract links from entities
    chart_link = None
    buy_link = None
    message_link = None
    for entity in entities:
        if entity.type == MessageEntity.TEXT_LINK:
            link = entity.url
            if link.startswith("https://gmgn.ai/sol/token/rLkfkJiz_"):
                contract_number = link.split("rLkfkJiz_")[1]
                chart_link = link
                buy_link = f"https://t.me/GMGN_sol_bot?start=i_rLkfkJiz_c_{contract_number}"
            elif "t.me" in link:
                message_link = link

    if not chart_link or not buy_link:
        print("[DEBUG] Failed to extract links")
        return None

    print(f"[DEBUG] Chart link: {chart_link}")
    print(f"[DEBUG] Buy link: {buy_link}")
    print(f"[DEBUG] Message link: {message_link}")

    contract_address = contract_number
    print(f"[DEBUG] Contract address: {contract_address}")

    return {
        'contract_address': contract_address,
        'name': name,
        'ticker': ticker,
        'chart_link': chart_link,
        'buy_link': buy_link,
        'message_link': message_link
    }

# Handle incoming messages
async def handle_message(update: Update, context):
    print("[DEBUG] Received update")

    message = update.message
    if not message:
        print("[DEBUG] No message in update")
        return

    print(f"[DEBUG] Chat ID: {message.chat.id}, Thread ID: {message.message_thread_id}")
    print(f"[DEBUG] Full message object:\n{message.to_dict()}")

    if message.chat.id != GROUP_ID:
        print("[DEBUG] Message not in tracked group")
        return

    if message.message_thread_id not in TRACKED_TOPICS:
        print(f"[DEBUG] Thread ID {message.message_thread_id} not tracked")
        return

    if not message.text:
        print("[DEBUG] No text content")
        return

    text = message.text
    entities = message.entities or []
    print(f"[DEBUG] Raw message content:\n{text}")

    parsed = parse_message(text, entities)
    if not parsed:
        print("[DEBUG] Failed to parse message")
        return

    # Fetch current market cap from API
    current_mc = await get_current_market_cap(parsed['contract_address'])
    if current_mc is not None:
        parsed['market_cap'] = current_mc
        print(f"[DEBUG] Using API market cap: {current_mc}")
    else:
        # Fallback to parsing market cap from message
        mc_match = re.search(r'MC:\s*\$([\d\.]+)([km]?)', text, re.IGNORECASE)
        if mc_match:
            mc_value = float(mc_match.group(1))
            mc_unit = mc_match.group(2).lower()
            if mc_unit == 'k':
                mc_value *= 1000
            elif mc_unit == 'm':
                mc_value *= 1000000
            parsed['market_cap'] = mc_value
            print(f"[DEBUG] Using parsed market cap (API failed): {mc_value}")
        else:
            print("[DEBUG] No market cap found, skipping")
            return

    print(f"[DEBUG] Parsed token: {parsed}")

    contract_address = parsed['contract_address']
    print(f"[DEBUG] Processing token: {contract_address}")

    async with aiosqlite.connect(DB_FILE) as db:
        cursor = await db.execute('SELECT first_seen_timestamp FROM tokens WHERE contract_address = ?', (contract_address,))
        row = await cursor.fetchone()

        if row:
            existing_timestamp = row[0]  # Float Unix timestamp
            print(f"[DEBUG] Existing token seen at: {existing_timestamp}")
            if message.date.timestamp() < existing_timestamp:
                print("[DEBUG] Updating with earlier timestamp")
                await db.execute('''
                    UPDATE tokens SET
                        name = ?, ticker = ?, initial_market_cap = ?, first_topic = ?,
                        chart_link = ?, buy_link = ?, message_link = ?, first_seen_timestamp = ?
                    WHERE contract_address = ?
                ''', (
                    parsed['name'], parsed['ticker'], parsed['market_cap'], str(message.message_thread_id),
                    parsed['chart_link'], parsed['buy_link'], parsed['message_link'], message.date.timestamp(), contract_address
                ))
                await db.commit()
        else:
            print("[DEBUG] Inserting new token")
            await db.execute('''
                INSERT INTO tokens (
                    contract_address, name, ticker, initial_market_cap, first_topic,
                    chart_link, buy_link, message_link, first_seen_timestamp
                ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
            ''', (
                contract_address, parsed['name'], parsed['ticker'], parsed['market_cap'],
                str(message.message_thread_id), parsed['chart_link'], parsed['buy_link'], parsed['message_link'], message.date.timestamp()
            ))
            await db.commit()

# Fetch current market cap using DexScreener API
async def get_current_market_cap(contract):
    print(f"[DEBUG] Fetching market cap for {contract}")
    url = DEXSCREENER_API.format(contract=contract)
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(url, timeout=5) as response:
                if response.status == 200:
                    data = await response.json()
                    if "pairs" in data and data["pairs"]:
                        pair = data["pairs"][0]
                        market_cap = pair.get("marketCap")
                        if market_cap:
                            print(f"[DEBUG] Market cap for {contract}: {market_cap}")
                            return float(market_cap)
                    print(f"[DEBUG] No valid market cap data for {contract}")
                    return None
                return None
    except (aiohttp.ClientError, asyncio.TimeoutError, ValueError) as e:
        print(f"[ERROR] Error fetching DexScreener data: {e}")
        return None

# Periodically check market caps and post alerts
async def check_market_caps(application):
    while True:
        print("[DEBUG] Starting market cap check")
        async with aiosqlite.connect(DB_FILE) as db:
            db.row_factory = aiosqlite.Row
            cursor = await db.execute('SELECT * FROM tokens')
            tokens = await cursor.fetchall()
            print(f"[DEBUG] Fetched {len(tokens)} tokens from database")
            for token in tokens:
                current_mc = await get_current_market_cap(token['contract_address'])
                if current_mc is None:
                    continue
                print(f"[DEBUG] Current MC for {token['contract_address']}: {current_mc}")
                if current_mc <= 16000:  # Market cap drop check
                    print(f"[DEBUG] Deleting token {token['contract_address']} due to market cap drop")
                    await db.execute('DELETE FROM tokens WHERE contract_address = ?', (token['contract_address'],))
                    await db.commit()
                else:
                    # Calculate the current multiple
                    print(f"[DEBUG] Calculating multiple: current_mc={current_mc}, initial_mc={token['initial_market_cap']}")
                    multiple = int(current_mc // token['initial_market_cap'])
                    print(f"[DEBUG] Multiple: {multiple}")
                    if multiple > token['last_alerted_multiple']:
                        # Send only one message for the current highest multiple
                        message = (
                            f"🚀 {token['ticker']} MC: {format_market_cap(current_mc)} 💸 did {multiple}x since it was first alerted at {format_market_cap(token['initial_market_cap'])} 💥 on Vaultboy Private Group\n"
                            f"💎 {token['contract_address']} - <a href=\"{token['chart_link']}\">📊 CHART</a> | <a href=\"{token['buy_link']}\">🔥 BUY</a> |<a href=https://t.me/gmgnverify_bot>🔥 JOIN VB PRIVATE 🔥</a>"
                        )
                        print(f"[DEBUG] Sending alert for {token['ticker']}: {multiple}x")
                        await application.bot.send_message(chat_id=GROUP_ID, message_thread_id=ALERT_TOPIC_ID, text=message, parse_mode='HTML', disable_web_page_preview=True)
                        # Update last_alerted_multiple to the current multiple
                        await db.execute('UPDATE tokens SET last_alerted_multiple = ? WHERE contract_address = ?', (multiple, token['contract_address']))
                        await db.commit()
        print(f"[DEBUG] Sleeping for {CHECK_INTERVAL} seconds")
        await asyncio.sleep(CHECK_INTERVAL)

# Setup and run the bot
async def setup_bot():
    await init_db()
    application = Application.builder().token(BOT_TOKEN).build()
    application.add_handler(MessageHandler(filters.ALL, handle_message))
    print("[DEBUG] Bot is now polling...")
    task = asyncio.create_task(check_market_caps(application))
    return application, task

if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    application, task = loop.run_until_complete(setup_bot())
    try:
        application.run_polling()
    finally:
        task.cancel()
        loop.run_until_complete(task)
        loop.close()
