import os
import re
import json
import requests
import asyncio
from datetime import datetime
import io

from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Application, CommandHandler, CallbackContext, MessageHandler, filters

# ----------------- Configuration -----------------
SOURCE_GROUP_ID = -1002596386377
SOURCE_TOPIC_IDS = [864]
TELEGRAM_BOT_TOKEN = '7877860335:AAGU-3RaQTaXO74HZlmyy6YsVr35THWGv08'
GROUPS_FILE = "groups.json"

# ----------------- API Configuration -----------------
API_ENDPOINTS = {
    "dexscreener": "https://api.dexscreener.com/latest/dex/tokens/{contract}"
}

# ----------------- Group Management -----------------
def load_groups():
    if os.path.exists(GROUPS_FILE):
        try:
            with open(GROUPS_FILE, "r") as file:
                data = json.load(file)
                if data and isinstance(data, list) and not isinstance(data[0], dict):
                    data = [{"id": group, "title": "Unknown", "active": True} for group in data]
                    save_groups(data)
                return data
        except json.JSONDecodeError:
            return []
    return []

def save_groups(groups):
    with open(GROUPS_FILE, "w") as file:
        json.dump(groups, file)
    print(f"[DEBUG] Saved groups: {groups}")

def add_group(chat):
    groups = load_groups()
    if not any(g["id"] == chat.id for g in groups):
        groups.append({"id": chat.id, "title": chat.title or "Private Chat", "active": True})
        save_groups(groups)
        print(f"[DEBUG] Added chat {chat.title or 'Private'} (ID: {chat.id})")
    else:
        print(f"[DEBUG] Chat {chat.title or 'Private'} (ID: {chat.id}) already tracked")

def remove_group(chat_id):
    groups = load_groups()
    new_groups = [g for g in groups if g["id"] != chat_id]
    if len(new_groups) < len(groups):
        save_groups(new_groups)
        print(f"[DEBUG] Removed chat with ID {chat_id}")
    else:
        print(f"[DEBUG] Chat with ID {chat_id} not found")

def toggle_group_status(chat_id, active):
    groups = load_groups()
    for group in groups:
        if group["id"] == chat_id:
            group["active"] = active
            save_groups(groups)
            status = "active" if active else "inactive"
            print(f"[DEBUG] Set chat {group['title']} (ID: {chat_id}) to {status}")
            return True
    return False

# ----------------- API Functions -----------------
def fetch_data(contract):
    results = {}
    print(f"[DEBUG] Starting API query for contract: {contract}")

    url = API_ENDPOINTS["dexscreener"].format(contract=contract)
    print(f"[DEBUG] Querying endpoint: dexscreener -> URL: {url}")
    try:
        response = requests.get(url)
        response.raise_for_status()
        data = response.json()

        if "pairs" in data and data["pairs"]:
            pair = data["pairs"][0]  # Use the first pair (usually the most relevant)
            results["price_usd"] = pair.get("priceUsd", "N/A")
            results["market_cap"] = pair.get("marketCap", "N/A")
            results["fdv"] = pair.get("fdv", "N/A")  # Fully Diluted Valuation
            results["liquidity_usd"] = pair["liquidity"].get("usd", "N/A") if pair.get("liquidity") else "N/A"
            results["sol_pooled"] = pair["liquidity"].get("quote", "N/A") if pair.get("liquidity") else "N/A"
            results["volume_24h"] = pair["volume"].get("h24", "N/A") if pair.get("volume") else "N/A"
            results["pair_created_at"] = pair.get("pairCreatedAt", 0) / 1000
            results["price_change"] = pair.get("priceChange", {})
            results["pair_address"] = pair.get("pairAddress", "N/A")
            info = pair.get("info", {})
            results["dexscreener_socials"] = {s["type"]: s["url"] for s in info.get("socials", [])}
            results["dexscreener_website"] = info.get("websites", [{}])[0].get("url", "N/A")
        else:
            print(f"[DEBUG] No pairs found in Dexscreener response for {contract}")
    except requests.exceptions.RequestException as e:
        print(f"Error fetching {url}: {e}")

    print(f"[DEBUG] Final results: {json.dumps(results, indent=2)[:200]}...")
    return results

def format_compact_message(api_data):
    days_ago = int((datetime.now().timestamp() - api_data.get("pair_created_at", 0)) / (24 * 3600)) if api_data.get("pair_created_at") else "N/A"
    mc = api_data.get("market_cap", "N/A")
    mc_str = f"${mc / 1_000_000:.1f}M" if isinstance(mc, (int, float)) else "N/A"
    fdv = api_data.get("fdv", "N/A")
    fdv_str = f"${fdv / 1_000_000:.1f}M" if isinstance(fdv, (int, float)) else "N/A"
    liq = api_data.get("liquidity_usd", "N/A")
    liq_str = f"${liq / 1000:.1f}K" if isinstance(liq, (int, float)) else "N/A"
    vol_24h = api_data.get("volume_24h", "N/A")
    vol_str = f"${vol_24h / 1000:.1f}K" if isinstance(vol_24h, (int, float)) else "N/A"
    website = f"[Website]({api_data.get('dexscreener_website', 'N/A')})" if api_data.get("dexscreener_website") != "N/A" else ""
    telegram = f"[Telegram]({api_data.get('dexscreener_socials', {}).get('telegram', 'N/A')})" if api_data.get("dexscreener_socials", {}).get("telegram") != "N/A" else ""
    twitter = f"[Twitter/X]({api_data.get('dexscreener_socials', {}).get('twitter', 'N/A')})" if api_data.get("dexscreener_socials", {}).get("twitter") != "N/A" else ""

    message = f"""
├ **Create:** {days_ago}d ago
├ **Price:** ${api_data.get("price_usd", "N/A")}
├ **MC:** {mc_str}
├ **FDV:** {fdv_str}
├ **Liq:** {liq_str} / SOL pooled: {api_data.get("sol_pooled", "N/A")}
├ **Vol 24h:** {vol_str}
├ **Pair:** {api_data.get("pair_address", "N/A")}
├ **Change:** 5m: {api_data.get("price_change", {}).get('m5', "N/A")}% | 1h: {api_data.get("price_change", {}).get('h1', "N/A")}% | 6h: {api_data.get("price_change", {}).get('h6', "N/A")}% | 24h: {api_data.get("price_change", {}).get('h24', "N/A")}%

🔗 {website} {telegram} {twitter}
""".strip()
    return message

async def process_message(message: Update.message, context: CallbackContext):
    groups = load_groups()
    text = message.text
    print(f"[DEBUG] Processing message: {text[:100]}...")

    contract = None
    contract_match = re.search(r'https://gmgn\.ai/sol/token/rLkfkJiz_([A-Za-z0-9]+)', text)
    if contract_match:
        contract = contract_match.group(1)
        print(f"[DEBUG] Contract extracted from text: {contract}")
    elif message.entities:
        for entity in message.entities:
            if entity.type == 'text_link' and 'gmgn.ai/sol/token/rLkfkJiz_' in entity.url:
                contract_match = re.search(r'https://gmgn\.ai/sol/token/rLkfkJiz_([A-Za-z0-9]+)', entity.url)
                if contract_match:
                    contract = contract_match.group(1)
                    print(f"[DEBUG] Contract extracted from entity: {contract}")
                    break

    buttons_data = []
    if message.entities:
        for entity in message.entities:
            if entity.type == 'text_link':
                url = entity.url
                if 'gmgn.ai/sol/token/rLkfkJiz_' in url:
                    buttons_data.append(("CHART", url))
                elif 't.me/GMGN' in url:
                    buttons_data.append(("SNIPE/BUY", url))
    if not buttons_data:
        pattern = r'([^|]+?)\s*\((https?://[^\)]+)\)'
        buttons_data = re.findall(pattern, text)
    print(f"[DEBUG] Buttons extracted: {buttons_data}")

    keyboard = InlineKeyboardMarkup([[InlineKeyboardButton(name, url=url) for name, url in buttons_data]])
    edited_text = text
    media_content_raw = None
    content_type = None
    filename = None

    if contract:
        try:
            loop = asyncio.get_running_loop()
            api_data = await loop.run_in_executor(None, fetch_data, contract)
            formatted_api = format_compact_message(api_data)
            edited_text += "\n\n" + formatted_api
            edited_text = edited_text.replace("📊CHART | 🎯BUY", "").strip()
            media_url = f"https://dd.dexscreener.com/ds-data/tokens/solana/{contract}/header.png"
            print(f"[DEBUG] Media URL set: {media_url}")

            headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'}
            media_response = requests.get(media_url, headers=headers, allow_redirects=True, timeout=5)
            content_type = media_response.headers.get('Content-Type', '')
            print(f"[DEBUG] Media URL GET response - Status: {media_response.status_code}, Content-Type: {content_type}, Content-Length: {media_response.headers.get('Content-Length', 'N/A')}")

            if media_response.status_code == 200 and media_response.content:
                media_content_raw = media_response.content
                if content_type in ['image/jpeg', 'image/png'] or (content_type == 'image/webp' and len(media_content_raw) < 100000):
                    filename = "header.jpg" if content_type == 'image/jpeg' else "header.png" if content_type == 'image/png' else "header.webp"
                    print(f"[DEBUG] Media downloaded successfully - Size: {len(media_content_raw)} bytes, Filename: {filename}")
                else:
                    print(f"[DEBUG] Media is not a static image (Content-Type: {content_type}, Size: {len(media_content_raw)} bytes), falling back to text")
                    media_content_raw = None
            else:
                print(f"[DEBUG] Media URL invalid (Status: {media_response.status_code}), falling back to text")
                media_content_raw = None
        except Exception as e:
            print(f"[DEBUG] Error downloading media or fetching API data: {e}")
            media_content_raw = None

    for group in groups:
        if group["id"] == SOURCE_GROUP_ID or not group["active"]:
            continue
        try:
            if media_content_raw:
                media_content = io.BytesIO(media_content_raw)
                media_content.seek(0)
                await context.bot.send_photo(
                    chat_id=group["id"],
                    photo=media_content,
                    filename=filename,
                    caption=edited_text,
                    reply_markup=keyboard,
                    parse_mode='Markdown'
                )
                print(f"[DEBUG] Sent photo with caption to chat {group['title']} (ID: {group['id']})")
                media_content.close()
            else:
                await context.bot.send_message(
                    chat_id=group["id"],
                    text=edited_text,
                    reply_markup=keyboard,
                    parse_mode='Markdown',
                    disable_web_page_preview=True
                )
                print(f"[DEBUG] Fallback: Sent text to chat {group['title']} (ID: {group['id']})")
        except Exception as e:
            print(f"[DEBUG] Error sending to chat {group['title']} (ID: {group['id']}): {e}")
            if "Forbidden" in str(e):
                remove_group(group["id"])

async def message_handler(update: Update, context: CallbackContext):
    if update.message:
        if update.message.chat.id != SOURCE_GROUP_ID:
            return
        if not hasattr(update.message, 'message_thread_id'):
            return
        if update.message.message_thread_id not in SOURCE_TOPIC_IDS:
            return
        await process_message(update.message, context)

async def start(update: Update, context: CallbackContext):
    chat = update.message.chat
    add_group(chat)
    toggle_group_status(chat.id, True)
    groups = load_groups()
    group_info = f"Tracking {len([g for g in groups if g['active']])} active chats"
    print(f"[DEBUG] {group_info}")
    keyboard = InlineKeyboardMarkup([
        [InlineKeyboardButton("Vaultboy Alerts", url="https://t.me/vaultboyportal")],
        [InlineKeyboardButton("GMGN Sniper", url="https://t.me/US_GMGNBOT?start=i_rLkfkJiz")]
    ])
    await update.message.reply_text(
        "Bot started! Forwarding Vaultboy VB1 multibuy alerts.\nUse /stop to stop receiving messages in this chat.",
        reply_markup=keyboard
    )

async def stop(update: Update, context: CallbackContext):
    chat = update.message.chat
    if toggle_group_status(chat.id, False):
        await update.message.reply_text(
            "Bot stopped posting in this chat. Use /start to resume."
        )
    else:
        groups = load_groups()
        if not any(g["id"] == chat.id for g in groups):
            groups.append({"id": chat.id, "title": chat.title or "Private Chat", "active": False})
            save_groups(groups)
            print(f"[DEBUG] Added chat {chat.title or 'Private'} (ID: {chat.id}) as inactive")
        await update.message.reply_text(
            "Bot stopped posting in this chat. Use /start to resume."
        )

async def handle_new_chat(update: Update, context: CallbackContext):
    chat = update.message.chat
    if chat.type in ["group", "supergroup", "private"]:
        add_group(chat)

async def handle_left_group(update: Update, context: CallbackContext):
    chat = update.message.chat
    if chat.type in ["group", "supergroup"]:
        remove_group(chat.id)

def main():
    application = Application.builder().token(TELEGRAM_BOT_TOKEN).build()
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("stop", stop))
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS | filters.StatusUpdate.CHAT_CREATED, handle_new_chat))
    application.add_handler(MessageHandler(filters.StatusUpdate.LEFT_CHAT_MEMBER, handle_left_group))
    application.add_handler(MessageHandler(filters.ALL & ~filters.COMMAND, message_handler))
    print("[DEBUG] Starting bot...")
    application.run_polling()

if __name__ == "__main__":
    main()
