import os
import re
import json
import requests
import asyncio
from datetime import datetime
import io

from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import Application, CommandHandler, CallbackContext, MessageHandler, filters

# ----------------- Configuration -----------------
SOURCE_GROUP_ID = -1002134751475  # Updated to match bsc alerts group from second bot
SOURCE_TOPIC_IDS = [2049077]      # bsc topic ID from CHAIN_TOPICS in second bot
TELEGRAM_BOT_TOKEN = '7756616669:AAEYHVgkdx2u4dVdt17QXyR5oLoFt3WxBvk'
GROUPS_FILE = "groups.json"

# ----------------- API Configuration -----------------
bsc_URL = "https://gmgn.ai"
GMGN_ENDPOINTS = {
    "security": "/api/v1/mutil_window_token_security_launchpad/{chain}/{contract}",
    "stats": "/api/v1/token_stat/{chain}/{contract}",
}
DEXSCREENER_API = "https://api.dexscreener.com/latest/dex/tokens/{contract}"
HONEYPOT_API = "https://api.honeypot.is/v2/IsHoneypot?address={contract}&chain={chain}"

# ----------------- Group Management -----------------
def load_groups():
    """Load group data from groups.json, ensuring proper structure."""
    if os.path.exists(GROUPS_FILE):
        try:
            with open(GROUPS_FILE, "r") as file:
                data = json.load(file)
                if not data:
                    return []
                if isinstance(data, list) and data and isinstance(data[0], dict):
                    for group in data:
                        if "active" not in group:
                            group["active"] = True
                    save_groups(data)
                elif isinstance(data, list) and data and not isinstance(data[0], dict):
                    data = [{"id": group, "title": "Unknown", "active": True} for group in data]
                    save_groups(data)
                return data
        except json.JSONDecodeError:
            return []
    return []

def save_groups(groups):
    """Save group data to groups.json."""
    with open(GROUPS_FILE, "w") as file:
        json.dump(groups, file)
    print(f"[DEBUG] Saved groups: {groups}")

def add_group(chat):
    """Add a new group or chat to the tracking list."""
    groups = load_groups()
    if not any(g["id"] == chat.id for g in groups):
        groups.append({"id": chat.id, "title": chat.title or "Private Chat", "active": True})
        save_groups(groups)
        print(f"[DEBUG] Added chat {chat.title or 'Private'} (ID: {chat.id})")
    else:
        print(f"[DEBUG] Chat {chat.title or 'Private'} (ID: {chat.id}) already tracked")

def remove_group(chat_id):
    """Remove a group or chat from the tracking list."""
    groups = load_groups()
    new_groups = [g for g in groups if g["id"] != chat_id]
    if len(new_groups) < len(groups):
        save_groups(new_groups)
        print(f"[DEBUG] Removed chat with ID {chat_id}")
    else:
        print(f"[DEBUG] Chat with ID {chat_id} not found")

def toggle_group_status(chat_id, active):
    """Toggle the active status of a group or chat."""
    groups = load_groups()
    for group in groups:
        if group["id"] == chat_id:
            group["active"] = active
            save_groups(groups)
            status = "active" if active else "inactive"
            print(f"[DEBUG] Set chat {group['title']} (ID: {chat_id}) to {status}")
            return True
    return False

# ----------------- API Functions -----------------
def is_valid_token(chain, contract):
    """Validate a bsc token using Honeypot and GMGN security checks."""
    if chain != "bsc":
        print(f"[DEBUG] Skipping validation for non-bsc chain: {chain}")
        return True

    # Honeypot API check
    url = HONEYPOT_API.format(contract=contract, chain=chain)
    try:
        response = requests.get(url, timeout=5)
        data = response.json()
        honeypot_result = data.get("honeypotResult", {}).get("isHoneypot", False)
        contract_code = data.get("contractCode", {})
        open_source = contract_code.get("openSource", True)
        is_proxy = contract_code.get("isProxy", False)
        has_proxy_calls = contract_code.get("hasProxyCalls", False)
        if honeypot_result or not open_source or is_proxy or has_proxy_calls:
            print(f"[DEBUG] Token {contract} failed Honeypot check: {data}")
            return False
    except Exception as e:
        print(f"[DEBUG] Honeypot API error: {e}")
        return False

    # GMGN security check
    try:
        url = bsc_URL + GMGN_ENDPOINTS["security"].format(chain=chain, contract=contract)
        response = requests.get(url, timeout=5)
        data = response.json()["data"]["security"]
        if data.get("is_honeypot", False) or not (data.get("is_renounced", False) or data.get("renounced_mint", False)):
            print(f"[DEBUG] Token {contract} failed GMGN security check")
            return False
    except Exception as e:
        print(f"[DEBUG] GMGN security API error: {e}")
        return False

    return True

def fetch_data(chain, contract):
    """Fetch token data from Dexscreener and GMGN APIs with fallbacks for missing data."""
    results = {}
    print(f"[DEBUG] Fetching data for chain: {chain}, contract: {contract}")

    # GMGN Security API
    try:
        url = bsc_URL + GMGN_ENDPOINTS["security"].format(chain=chain, contract=contract)
        response = requests.get(url, timeout=5)
        data = response.json()["data"]["security"]
        results["is_honeypot"] = data.get("is_honeypot", False)
        results["renounced"] = data.get("is_renounced", False) or data.get("renounced_mint", False)
    except Exception as e:
        print(f"[DEBUG] Error fetching GMGN security data: {e}")

    # GMGN Stats API
    try:
        url = bsc_URL + GMGN_ENDPOINTS["stats"].format(chain=chain, contract=contract)
        response = requests.get(url, timeout=5)
        data = response.json()["data"]
        results["holders"] = data.get("holder_count", "N/A")
    except Exception as e:
        print(f"[DEBUG] Error fetching GMGN stats: {e}")

    # DexScreener API with fallback for no pairs
    try:
        url = DEXSCREENER_API.format(contract=contract)
        response = requests.get(url, timeout=5)
        data = response.json()
        if "pairs" in data and data["pairs"]:  # Check if pairs exist and are non-empty
            pair = data["pairs"][0]
            results["price_usd"] = pair.get("priceUsd", "N/A")
            results["market_cap"] = float(pair.get("marketCap", "0")) if pair.get("marketCap") else "N/A"
            results["liquidity_usd"] = float(pair["liquidity"].get("usd", "0")) if pair["liquidity"].get("usd") else "N/A"
            results["pair_created_at"] = pair.get("pairCreatedAt", 0) / 1000  # Convert milliseconds to seconds
            results["price_change"] = pair.get("priceChange", {})
            info = pair.get("info", {})
            results["website"] = info.get("websites", [{}])[0].get("url", "N/A")
            socials = {s["type"]: s["url"] for s in info.get("socials", [])}
            results["twitter"] = socials.get("twitter", "N/A")
            results["telegram"] = socials.get("telegram", "N/A")
        else:
            print("[DEBUG] No trading pairs found in DexScreener data")
            results["price_usd"] = "N/A"
            results["market_cap"] = "N/A"
            results["liquidity_usd"] = "N/A"
            results["pair_created_at"] = "N/A"  # Fallback to "N/A" if no pairs
            results["price_change"] = {}
            results["website"] = "N/A"
            results["twitter"] = "N/A"
            results["telegram"] = "N/A"
    except Exception as e:
        print(f"[DEBUG] Error fetching DexScreener data: {e}")
        results["price_usd"] = "N/A"
        results["market_cap"] = "N/A"
        results["liquidity_usd"] = "N/A"
        results["pair_created_at"] = "N/A"
        results["price_change"] = {}
        results["website"] = "N/A"
        results["twitter"] = "N/A"
        results["telegram"] = "N/A"

    return results

def format_compact_message(api_data):
    """Format a compact message with token data for forwarding, handling non-numeric pair_created_at."""
    pair_created_at = api_data.get("pair_created_at", "N/A")
    if pair_created_at != "N/A" and isinstance(pair_created_at, (int, float)):
        days_ago = int((datetime.now().timestamp() - pair_created_at) / (24 * 3600))
    else:
        days_ago = "N/A"

    mc = api_data.get("market_cap", "N/A")
    mc_str = f"${mc / 1_000_000:.1f}M" if isinstance(mc, (int, float)) else "N/A"
    liq = api_data.get("liquidity_usd", "N/A")
    liq_str = f"${liq / 1000:.1f}K" if isinstance(liq, (int, float)) else "N/A"
    renounced = "✅" if api_data.get("renounced", False) else "❌"
    social_links = []
    if api_data.get("website", "N/A") != "N/A":
        social_links.append(f"[Website]({api_data['website']})")
    if api_data.get("twitter", "N/A") != "N/A":
        social_links.append(f"[Twitter]({api_data['twitter']})")
    if api_data.get("telegram", "N/A") != "N/A":
        social_links.append(f"[Telegram]({api_data['telegram']})")
    social_section = " | ".join(social_links) if social_links else "None"

    message = f"""
├ **Created:** {days_ago}d ago
├ **Price:** ${api_data.get("price_usd", "N/A")}
├ **MC:** {mc_str}
├ **Liq:** {liq_str}
├ **Renounced:** {renounced}
├ **Holders:** {api_data.get("holders", "N/A")}
├ **Change:** 5m: {api_data.get("price_change", {}).get('m5', "N/A")}% | 1h: {api_data.get("price_change", {}).get('h1', "N/A")}% | 24h: {api_data.get("price_change", {}).get('h24', "N/A")}%
🔗 {social_section}
""".strip()
    return message

async def process_message(message: Update.message, context: CallbackContext):
    """Process incoming messages, validate tokens, and forward to groups."""
    groups = load_groups()
    text = message.text
    print(f"[DEBUG] Processing message: {text[:100]}...")

    # Remove "🎯BUY|📊CHART" from the message text
    text = re.sub(r'🎯BUY\|📊CHART', '', text).strip()

    # Extract contract from CHART URL
    contract = None
    for entity in message.entities:
        if entity.type == 'text_link' and 'gmgn.ai/bsc/token/rLkfkJiz_' in entity.url:
            contract_match = re.search(r'https://gmgn\.ai/bsc/token/rLkfkJiz_([A-Za-z0-9]+)', entity.url)
            if contract_match:
                contract = contract_match.group(1)
                print(f"[DEBUG] Contract extracted: {contract}")
                break

    if not contract:
        print("[DEBUG] No bsc contract address found")
        return

    # Validate token
    loop = asyncio.get_running_loop()
    valid = await loop.run_in_executor(None, is_valid_token, "bsc", contract)
    if not valid:
        print(f"[DEBUG] Token {contract} failed validity checks, dropping message")
        return

    # Fetch token data
    api_data = await loop.run_in_executor(None, fetch_data, "bsc", contract)
    if not api_data:
        print(f"[DEBUG] Failed to fetch data for {contract}")
        return

    # Extract buttons
    buttons_data = []
    for entity in message.entities:
        if entity.type == 'text_link':
            url = entity.url
            if 'gmgn.ai/bsc/token/rLkfkJiz_' in url:
                buttons_data.append(("CHART", url))
            elif 't.me/GMGN_bsc_bot' in url:
                buttons_data.append(("BUY", url))
    print(f"[DEBUG] Buttons extracted: {buttons_data}")
    keyboard = InlineKeyboardMarkup([[InlineKeyboardButton(name, url=url) for name, url in buttons_data]])

    # Format message
    edited_text = text + "\n\n" + format_compact_message(api_data)

    # Fetch media (optional)
    media_content_raw = None
    filename = None
    media_url = f"https://dd.dexscreener.com/ds-data/tokens/bsc/{contract}/header.png"
    try:
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'}
        response = requests.get(media_url, headers=headers, allow_redirects=True, timeout=5)
        content_type = response.headers.get('Content-Type', '')
        if response.status_code == 200 and content_type in ['image/jpeg', 'image/png']:
            media_content_raw = response.content
            filename = "header.jpg" if content_type == 'image/jpeg' else "header.png"
            print(f"[DEBUG] Media downloaded: {filename}")
    except Exception as e:
        print(f"[DEBUG] Error fetching media: {e}")

    # Forward to all active groups and channels
    for group in groups:
        if group["id"] == SOURCE_GROUP_ID or not group["active"]:
            continue
        try:
            if media_content_raw:
                media_content = io.BytesIO(media_content_raw)
                media_content.seek(0)
                await context.bot.send_photo(
                    chat_id=group["id"],
                    photo=media_content,
                    filename=filename,
                    caption=edited_text,
                    reply_markup=keyboard,
                    parse_mode='Markdown'
                )
                print(f"[DEBUG] Sent photo to {group['title']} (ID: {group['id']})")
                media_content.close()
            else:
                await context.bot.send_message(
                    chat_id=group["id"],
                    text=edited_text,
                    reply_markup=keyboard,
                    parse_mode='Markdown',
                    disable_web_page_preview=True
                )
                print(f"[DEBUG] Sent text to {group['title']} (ID: {group['id']})")
        except Exception as e:
            print(f"[DEBUG] Error sending to {group['title']} (ID: {group['id']}): {e}")
            if "Forbidden" in str(e):
                remove_group(group["id"])

async def message_handler(update: Update, context: CallbackContext):
    """Handle incoming messages from the source group and topic."""
    if update.message:
        if update.message.chat.id != SOURCE_GROUP_ID:
            return
        if not hasattr(update.message, 'message_thread_id'):
            return
        if update.message.message_thread_id not in SOURCE_TOPIC_IDS:
            return
        await process_message(update.message, context)

async def start(update: Update, context: CallbackContext):
    """Start command to activate the bot in a chat."""
    chat = update.message.chat
    add_group(chat)
    toggle_group_status(chat.id, True)
    groups = load_groups()
    group_info = f"Tracking {len([g for g in groups if g['active']])} active chats"
    print(f"[DEBUG] {group_info}")
    keyboard = InlineKeyboardMarkup([
        [InlineKeyboardButton("Vaultboy Alerts", url="https://t.me/vaultboyportal")],
        [InlineKeyboardButton("GMGN Sniper", url="https://t.me/US_GMGNBOT?start=i_rLkfkJiz")]
    ])
    await update.message.reply_text(
        "Bot started! Forwarding Vaultboy BSC alerts.\nUse /stop to stop receiving messages in this chat.",
        reply_markup=keyboard
    )

async def stop(update: Update, context: CallbackContext):
    """Stop command to deactivate the bot in a chat."""
    chat = update.message.chat
    if toggle_group_status(chat.id, False):
        await update.message.reply_text(
            "Bot stopped posting in this chat. Use /start to resume."
        )
    else:
        groups = load_groups()
        if not any(g["id"] == chat.id for g in groups):
            groups.append({"id": chat.id, "title": chat.title or "Private Chat", "active": False})
            save_groups(groups)
            print(f"[DEBUG] Added chat {chat.title or 'Private'} (ID: {chat.id}) as inactive")
        await update.message.reply_text(
            "Bot stopped posting in this chat. Use /start to resume."
        )

async def handle_new_chat(update: Update, context: CallbackContext):
    """Handle bot being added to a new chat."""
    chat = update.message.chat
    if chat.type in ["group", "supergroup", "channel", "private"]:
        add_group(chat)

async def handle_left_group(update: Update, context: CallbackContext):
    """Handle bot being removed from a group."""
    chat = update.message.chat
    if chat.type in ["group", "supergroup", "channel"]:
        remove_group(chat.id)

def main():
    """Main function to start the bot."""
    application = Application.builder().token(TELEGRAM_BOT_TOKEN).build()
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("stop", stop))
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS | filters.StatusUpdate.CHAT_CREATED, handle_new_chat))
    application.add_handler(MessageHandler(filters.StatusUpdate.LEFT_CHAT_MEMBER, handle_left_group))
    application.add_handler(MessageHandler(filters.ALL & ~filters.COMMAND, message_handler))
    print("[DEBUG] Starting bot...")
    application.run_polling()

if __name__ == "__main__":
    main()
